@@ -1,6 +1,6 @@
 # This is the PyQt5 build script.
 #
-# Copyright (c) 2019 Riverbank Computing Limited <info@riverbankcomputing.com>
+# Copyright (c) 2020 Riverbank Computing Limited <info@riverbankcomputing.com>
 # 
 # This file is part of PyQt5.
 # 
@@ -33,7 +33,8 @@ class PyQt(PyQtProject):
     def __init__(self):
         """ Initialise the project. """
 
-        super().__init__(dunder_init=True, tag_prefix='Qt',
+        super().__init__(sip_module='PyQt5.sip', abi_version='12.8',
+                dunder_init=True, tag_prefix='Qt',
                 console_scripts=[
                     'pylupdate5 = PyQt5.pylupdate_main:main',
                     'pyrcc5 = PyQt5.pyrcc_main:main',
@@ -44,12 +45,13 @@ class PyQt(PyQtProject):
             QtQml, QAxContainer, QtAndroidExtras, QtBluetooth, QtDBus,
             QtDesigner, Enginio, QtHelp, QtMacExtras, QtMultimedia,
             QtMultimediaWidgets, QtNetworkAuth, QtNfc, QtOpenGL, QtPositioning,
-            QtLocation, QtPrintSupport, QtQuick, QtQuickWidgets,
+            QtLocation, QtPrintSupport, QtQuick, QtQuick3D, QtQuickWidgets,
             QtRemoteObjects, QtSensors, QtSerialPort, QtSql, QtSvg, QtTest,
-            QtWebChannel, QtWebKit, QtWebKitWidgets, QtWebSockets, QtWinExtras,
-            QtX11Extras, QtXml, QtXmlPatterns, _QOpenGLFunctions_2_0,
-            _QOpenGLFunctions_2_1, _QOpenGLFunctions_4_1_Core,
-            _QOpenGLFunctions_ES2, pylupdate, pyrcc]
+            QtTextToSpeech, QtWebChannel, QtWebKit, QtWebKitWidgets,
+            QtWebSockets, QtWinExtras, QtX11Extras, QtXml, QtXmlPatterns,
+            _QOpenGLFunctions_2_0, _QOpenGLFunctions_2_1,
+            _QOpenGLFunctions_4_1_Core, _QOpenGLFunctions_ES2, pylupdate,
+            pyrcc]
 
     def apply_user_defaults(self, tool):
         """ Set default values where needed. """
@@ -172,7 +174,7 @@ del find_qt
         self.bindings['QtCore'].tags.append(plattag)
 
         # Make sure the bindings are buildable.
-        self.update_buildable_bindings()
+        super().update(tool)
 
         # PyQtWebEngine needs to know if QtWebChannel is available.
         if 'QtWebChannel' not in self.bindings:
@@ -189,20 +191,29 @@ del find_qt
         installable.files.append(os.path.join(self.root_dir, 'pyuic', 'uic'))
         self.installables.append(installable)
 
+        # If any set of bindings is being built as a debug version then assume
+        # the plugins and DBus support should as well.
+        for bindings in self.bindings.values():
+            if bindings.debug:
+                others_debug = True
+                break
+        else:
+            others_debug = self.py_debug
+
         # Add the plugins.  For the moment we don't include them in wheels.
         # This may change when we improve the bundling of Qt.
         if tool in ('build', 'install'):
             if self.designer_plugin and 'QtDesigner' in self.bindings:
                 self._add_plugin('designer', "Qt Designer", 'pyqt5',
-                        'designer')
+                        'designer', others_debug)
 
             if self.qml_plugin and 'QtQml' in self.bindings:
                 self._add_plugin('qmlscene', "qmlscene", 'pyqt5qmlplugin',
-                        'PyQt5')
+                        'PyQt5', others_debug)
 
         # Add the dbus-python support.
         if self.dbus_python:
-            self._add_dbus()
+            self._add_dbus(others_debug)
 
     def _add_composite_module(self, tool):
         """ Add the bindings for the composite module. """
@@ -224,7 +235,7 @@ del find_qt
 
         self.bindings['Qt'].sip_file = sip_file
 
-    def _add_dbus(self):
+    def _add_dbus(self, debug):
         """ Add the dbus-python support. """
 
         self.progress(
@@ -287,11 +298,11 @@ del find_qt
         buildable.include_dirs.extend(dbus_inc_dirs)
         buildable.library_dirs.extend(dbus_lib_dirs)
         buildable.libraries.extend(dbus_libs)
-        buildable.debug = self.py_debug
+        buildable.debug = debug
 
         self.buildables.append(buildable)
 
-    def _add_plugin(self, name, user_name, target_name, target_subdir):
+    def _add_plugin(self, name, user_name, target_name, target_subdir, debug):
         """ Add a plugin to the project buildables. """
 
         builder = self.builder
@@ -301,15 +312,10 @@ del find_qt
             self.progress("The {0} plugin was disabled because a shared Python library couldn't be found.".format(user_name))
             return
 
-        # Where the plugin will be installed.  Check the user has write
-        # permission.
+        # Where the plugin will (eventually) be installed.
         target_plugin_dir = os.path.join(
                 builder.qt_configuration['QT_INSTALL_PLUGINS'], target_subdir)
 
-        if not os.access(target_plugin_dir, os.W_OK):
-            self.progress("The {0} plugin was disabled because you do not have write permission on '{1}'.".format(user_name, target_plugin_dir))
-            return
-
         # Create the buildable and add it to the builder.
         buildable = Buildable(self, name)
         self.buildables.append(buildable)
@@ -335,8 +341,7 @@ del find_qt
         with open(os.path.join(root_plugin_dir, name + '.pro-in')) as f:
             prj = f.read()
 
-        prj = prj.replace('@QTCONFIG@',
-                'debug' if self.py_debug else 'release')
+        prj = prj.replace('@QTCONFIG@', 'debug' if debug else 'release')
         prj = prj.replace('@PYINCDIR@',
                 builder.qmake_quote(self.py_include_dir))
         prj = prj.replace('@SIPINCDIR@', builder.qmake_quote(self.build_dir))
@@ -827,6 +832,17 @@ class QtQuick(PyQtBindings):
                 test_statement='new QQuickWindow()')
 
 
+class QtQuick3D(PyQtBindings):
+    """ The QtQuick3D bindings. """
+
+    def __init__(self, project):
+        """ Initialise the bindings. """
+
+        super().__init__(project, 'QtQuick3D', qmake_QT=['quick3d'],
+                test_headers=['qquick3d.h'],
+                test_statement='QQuick3D::idealSurfaceFormat()')
+
+
 class QtQuickWidgets(PyQtBindings):
     """ The QtQuickWidgets bindings. """
 
@@ -904,6 +920,18 @@ class QtTest(PyQtBindings):
                 test_headers=['QtTest'], test_statement='QTest::qSleep(0)')
 
 
+class QtTextToSpeech(PyQtBindings):
+    """ The QtTextToSpeech bindings. """
+
+    def __init__(self, project):
+        """ Initialise the bindings. """
+
+        super().__init__(project, 'QtTextToSpeech',
+                qmake_QT=['texttospeech', '-gui'],
+                test_headers=['QTextToSpeech'],
+                test_statement='new QTextToSpeech()')
+
+
 class QtWebChannel(PyQtBindings):
     """ The QtWebChannel bindings. """
 
